==> unity-ads-4.12.5-sources.jar: com/unity3d/ads/core/domain/AndroidInitializeBoldSDK.kt <==
package com.unity3d.ads.core.domain

import com.unity3d.ads.core.data.model.InitializationState
import com.unity3d.ads.core.data.repository.SessionRepository
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.INIT_FAILURE
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.INIT_STARTED
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.INIT_SUCCESS
import com.unity3d.ads.UnityAds.UnityAdsInitializationError.INTERNAL_ERROR
import com.unity3d.ads.core.domain.om.InitializeOMSDK
import com.unity3d.ads.core.data.manager.SDKPropertiesManager
import com.unity3d.ads.core.data.repository.DiagnosticEventRepository
import com.unity3d.ads.core.domain.events.EventObservers
import com.unity3d.ads.gatewayclient.GatewayClient
import com.unity3d.ads.core.data.manager.StorageManager
import com.unity3d.ads.core.data.model.OperationType
import com.unity3d.ads.core.data.model.exception.InitializationException
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.OPERATION
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.REASON
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.REASON_DEBUG
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.REASON_GATEWAY
import com.unity3d.ads.core.extensions.elapsedMillis
import com.unity3d.services.UnityAdsConstants
import com.unity3d.services.core.configuration.ConfigurationReader
import com.unity3d.services.core.log.DeviceLog
import kotlinx.coroutines.CoroutineDispatcher
import kotlinx.coroutines.withContext
import kotlinx.coroutines.withTimeout


import kotlin.time.ExperimentalTime
import kotlin.time.TimeMark
import kotlin.time.TimeSource

@OptIn(ExperimentalTime::class)

internal class AndroidInitializeBoldSDK(
    private val defaultDispatcher: CoroutineDispatcher,
    private val initializeOM: InitializeOMSDK,
    private val getInitializeRequest: GetInitializationRequest,
    private val getRequestPolicy: GetRequestPolicy,
    private val clearCache: ClearCache,
    private val handleGatewayInitializationResponse: HandleGatewayInitializationResponse,
    private val gatewayClient: GatewayClient,
    private val sessionRepository: SessionRepository,
    private val eventObservers: EventObservers,
    private val triggerInitializeListener: TriggerInitializeListener,
    private val sendDiagnosticEvent: SendDiagnosticEvent,
    private val diagnosticEventRepository: DiagnosticEventRepository,
    private val storageManager: StorageManager,
    private val legacyConfigurationReader: ConfigurationReader,
    private val sdkPropertiesManager: SDKPropertiesManager,
) : InitializeBoldSDK {

    override suspend fun invoke() = withContext(defaultDispatcher) {
        val startTime = TimeSource.Monotonic.markNow()

        try {
            withTimeout(UnityAdsConstants.Timeout.INIT_TIMEOUT_MS) {
                initializationStart()
                checkCanInitialize()
                val initializationRequest = getInitializeRequest()
                val requestPolicy = getRequestPolicy()
                val response = gatewayClient.request(request = initializationRequest, requestPolicy = requestPolicy, operationType = OperationType.INITIALIZATION)
                handleGatewayInitializationResponse(response.payload.initializationResponse)
            }
        } catch (e: Exception) {
            val failure = InitializationException.parseFrom(e)
            initializationFailure(startTime, failure)
            return@withContext
        }

        initializationSuccess(startTime)
    }

    private fun checkCanInitialize() {
        if (!sessionRepository.shouldInitialize) {
            throw InitializationException(message = MSG_GATEWAY_DENIED, reason = REASON_GATEWAY, reasonDebug = "!sessionRepository.shouldInitialize")
        }
    }

    private suspend fun initializationStart() {
        DeviceLog.debug("Unity Ads Initialization Start")
        sendDiagnosticEvent(event = INIT_STARTED)
        sessionRepository.initializationState = InitializationState.INITIALIZING
        eventObservers()
    }

    private suspend fun initializationSuccess(startTime: TimeMark) {
        DeviceLog.debug("Unity Ads Initialization Success")
        sendDiagnosticEvent(event = INIT_SUCCESS, value = startTime.elapsedMillis())
        storageManager.hasInitialized()
        initializeOM()
        clearCache()
        // We persist config and remove legacy config from disk so we can be ready for clean slate in case we switch back to non gateway flow.
        sessionRepository.persistNativeConfiguration()
        legacyConfigurationReader.currentConfiguration.deleteFromDisk()
        sessionRepository.initializationState = InitializationState.INITIALIZED
        sdkPropertiesManager.setInitialized(true)
        triggerInitializeListener.success()
        setupDiagnosticEvents()
    }

    private fun initializationFailure(startTime: TimeMark, e: InitializationException) {
        DeviceLog.debug("Unity Ads Initialization Failure: ${e.message}")
        sendDiagnosticEvent(event = INIT_FAILURE, value = startTime.elapsedMillis(), tags = getTags(e))
        sessionRepository.initializationState = InitializationState.FAILED
        sdkPropertiesManager.setInitialized(false)
        triggerInitializeListener.error(INTERNAL_ERROR, e.message)
        setupDiagnosticEvents()
    }

    private fun getTags(e: InitializationException): Map<String, String> {
        return buildMap {
            put(OPERATION, OperationType.INITIALIZATION.toString())
            put(REASON, e.reason)
            if (e.reasonDebug != null) {
                put(REASON_DEBUG, e.reasonDebug)
            }
        }
    }

    private fun setupDiagnosticEvents() {
        val config = sessionRepository.nativeConfiguration.diagnosticEvents
        diagnosticEventRepository.configure(config)
    }

    companion object {
        const val MSG_GATEWAY_DENIED = "Gateway communication failure"
        const val MSG_TIMEOUT = "Timeout"
        const val MSG_NETWORK = "Network"
        const val MSG_UNKNOWN = "Initialization failure"
    }
}

==> unity-ads-4.13.0-sources.jar: com/unity3d/ads/core/domain/AndroidInitializeBoldSDK.kt <==
package com.unity3d.ads.core.domain


import com.unity3d.ads.UnityAds.UnityAdsInitializationError.INTERNAL_ERROR
import com.unity3d.ads.core.data.manager.SDKPropertiesManager
import com.unity3d.ads.core.data.manager.StorageManager
import com.unity3d.ads.core.data.model.InitializationState
import com.unity3d.ads.core.data.model.OperationType
import com.unity3d.ads.core.data.model.exception.InitializationException
import com.unity3d.ads.core.data.repository.DiagnosticEventRepository
import com.unity3d.ads.core.data.repository.SessionRepository
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.INIT_FAILURE
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.INIT_STARTED
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.INIT_SUCCESS
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.IS_RETRY
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.OPERATION
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.REASON
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.REASON_DEBUG
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.REASON_GATEWAY
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.REASON_INVALID_GAME_ID
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.SOURCE
import com.unity3d.ads.core.domain.events.EventObservers
import com.unity3d.ads.core.domain.om.InitializeOMSDK
import com.unity3d.ads.core.extensions.elapsedMillis
import com.unity3d.ads.gatewayclient.GatewayClient
import com.unity3d.services.UnityAdsConstants
import com.unity3d.services.core.configuration.ConfigurationReader
import com.unity3d.services.core.log.DeviceLog
import gatewayprotocol.v1.UniversalResponseOuterClass.UniversalResponse
import kotlinx.coroutines.CoroutineDispatcher
import kotlinx.coroutines.withContext
import kotlinx.coroutines.withTimeout
import kotlin.time.ExperimentalTime
import kotlin.time.TimeMark
import kotlin.time.TimeSource

@OptIn(ExperimentalTime::class)

internal class AndroidInitializeBoldSDK(
    private val defaultDispatcher: CoroutineDispatcher,
    private val initializeOM: InitializeOMSDK,
    private val getInitializeRequest: GetInitializationRequest,
    private val getRequestPolicy: GetRequestPolicy,
    private val clearCache: ClearCache,
    private val handleGatewayInitializationResponse: HandleGatewayInitializationResponse,
    private val gatewayClient: GatewayClient,
    private val sessionRepository: SessionRepository,
    private val eventObservers: EventObservers,
    private val triggerInitializeListener: TriggerInitializeListener,
    private val sendDiagnosticEvent: SendDiagnosticEvent,
    private val diagnosticEventRepository: DiagnosticEventRepository,
    private val storageManager: StorageManager,
    private val legacyConfigurationReader: ConfigurationReader,
    private val sdkPropertiesManager: SDKPropertiesManager,
    private val getGameId: GetGameId,
) : InitializeBoldSDK {

    override suspend fun invoke(source: String) = withContext(defaultDispatcher) {
        val startTime = TimeSource.Monotonic.markNow()
        val isRetry = !sessionRepository.isFirstInitAttempt

        try {
            withTimeout(UnityAdsConstants.Timeout.INIT_TIMEOUT_MS) {
                initializationStart(source, isRetry)
                checkCanInitialize()
                val initializationRequest = getInitializeRequest()
                val requestPolicy = getRequestPolicy()
                val response = gatewayClient.request(request = initializationRequest, requestPolicy = requestPolicy, operationType = OperationType.INITIALIZATION)
                handleResponse(response)
            }
        } catch (e: Exception) {
            val failure = InitializationException.parseFrom(e)
            initializationFailure(startTime, failure, source, isRetry)
            return@withContext
        }

        initializationSuccess(startTime, source, isRetry)
    }

    private suspend fun handleResponse(response: UniversalResponse) {
        if (response.hasError()) {
            sessionRepository.shouldInitialize = false
            throw InitializationException(message = MSG_GATEWAY_DENIED, reason = REASON_GATEWAY, reasonDebug = response.error.errorText)
        }
        if (!response.hasPayload()) {
            throw InitializationException(message = MSG_GATEWAY_DENIED, reason = REASON_GATEWAY, reasonDebug = "No payload")
        }
        if (!response.payload.hasInitializationResponse()) {
            throw InitializationException(message = MSG_GATEWAY_DENIED, reason = REASON_GATEWAY, reasonDebug = "No initialization response")
        }
        handleGatewayInitializationResponse(response.payload.initializationResponse)
    }

    private fun checkCanInitialize() {
        if (!sessionRepository.shouldInitialize) {
            throw InitializationException(message = MSG_GATEWAY_DENIED, reason = REASON_GATEWAY, reasonDebug = "!sessionRepository.shouldInitialize")
        }

        val gameId = getGameId()
        if (gameId?.toIntOrNull() == null) {
            throw InitializationException(message = "gameId \"$gameId\" should be a number.", reason = REASON_INVALID_GAME_ID, reasonDebug = gameId)
        }
    }

    private suspend fun initializationStart(source: String, isRetry: Boolean) {
        DeviceLog.debug("Unity Ads Initialization Start")
        sdkPropertiesManager.setInitializationTime()
        sdkPropertiesManager.setInitializationTimeSinceEpoch()
        sendDiagnosticEvent(event = INIT_STARTED, tags = mapOf(SOURCE to source, IS_RETRY to isRetry.toString()))
        sessionRepository.initializationState = InitializationState.INITIALIZING
        eventObservers()
    }

    private suspend fun initializationSuccess(startTime: TimeMark, source: String, isRetry: Boolean) {
        DeviceLog.debug("Unity Ads Initialization Success")
        sendDiagnosticEvent(event = INIT_SUCCESS, value = startTime.elapsedMillis(), tags = mapOf(SOURCE to source, IS_RETRY to isRetry.toString()))
        storageManager.hasInitialized()
        initializeOM()
        clearCache()
        // We persist config and remove legacy config from disk so we can be ready for clean slate in case we switch back to non gateway flow.
        sessionRepository.persistNativeConfiguration()
        legacyConfigurationReader.currentConfiguration.deleteFromDisk()
        sessionRepository.initializationState = InitializationState.INITIALIZED
        sdkPropertiesManager.setInitialized(true)
        triggerInitializeListener.success()
        setupDiagnosticEvents()
    }

    private fun initializationFailure(
        startTime: TimeMark,
        e: InitializationException,
        source: String,
        isRetry: Boolean
    ) {
        DeviceLog.debug("Unity Ads Initialization Failure: ${e.message}")
        sendDiagnosticEvent(event = INIT_FAILURE, value = startTime.elapsedMillis(), tags = getTags(e, source, isRetry))
        sessionRepository.initializationState = InitializationState.FAILED
        sdkPropertiesManager.setInitialized(false)
        triggerInitializeListener.error(INTERNAL_ERROR, e.message)
        setupDiagnosticEvents()
    }

    private fun getTags(e: InitializationException, source: String, isRetry: Boolean): Map<String, String> {
        return buildMap {
            put(OPERATION, OperationType.INITIALIZATION.toString())
            put(REASON, e.reason)
            put(SOURCE, source)
            put(IS_RETRY, isRetry.toString())
            if (e.reasonDebug != null) {
                put(REASON_DEBUG, e.reasonDebug)
            }
        }
    }

    private fun setupDiagnosticEvents() {
        val config = sessionRepository.nativeConfiguration.diagnosticEvents
        diagnosticEventRepository.configure(config)
    }

    companion object {
        const val MSG_GATEWAY_DENIED = "Gateway communication failure"
        const val MSG_TIMEOUT = "Timeout"
        const val MSG_NETWORK = "Network"
        const val MSG_UNKNOWN = "Initialization failure"
    }
}
